@@ -62,6 +62,8 @@ import com.sun.jna.NativeMappedConverter;
 import com.sun.jna.Platform;
 import com.sun.jna.Pointer;
 import com.sun.jna.platform.win32.BaseTSD.SIZE_T;
+import com.sun.jna.platform.win32.BaseTSD.ULONG_PTR;
+import com.sun.jna.platform.win32.BaseTSD.ULONG_PTRByReference;
 import com.sun.jna.platform.win32.Ntifs.REPARSE_DATA_BUFFER;
 import com.sun.jna.platform.win32.Ntifs.SymbolicLinkReparseBuffer;
 import com.sun.jna.platform.win32.WinBase.FILETIME;
@@ -463,6 +465,60 @@ public class Kernel32Test extends TestCase {
         }
     }
 
+    public void testGetAndSetProcessAffinityMask() {
+        // Pseudo handle, no need to close. Has PROCESS_ALL_ACCESS right.
+        HANDLE pHandle = Kernel32.INSTANCE.GetCurrentProcess();
+        assertNotNull(pHandle);
+
+        ULONG_PTRByReference pProcessAffinity = new ULONG_PTRByReference();
+        ULONG_PTRByReference pSystemAffinity = new ULONG_PTRByReference();
+        assertTrue("Failed to get affinity masks.",
+                Kernel32.INSTANCE.GetProcessAffinityMask(pHandle, pProcessAffinity, pSystemAffinity));
+
+        long processAffinity = pProcessAffinity.getValue().longValue();
+        long systemAffinity = pSystemAffinity.getValue().longValue();
+
+        if (systemAffinity == 0) {
+            // Rare case for process to be running in multiple processor groups, where both
+            // systemAffinity and processAffinity are 0 and we can't do anything else.
+            assertEquals(
+                    "Both process and system affinity must be zero if this process is running in multiple processor groups",
+                    processAffinity, systemAffinity);
+        } else {
+            // Test current affinity
+            assertEquals("Process affinity must be a subset of system affinity", processAffinity,
+                    processAffinity & systemAffinity);
+            assertEquals("System affinity must be a superset of process affinity", systemAffinity,
+                    processAffinity | systemAffinity);
+
+            // Set affinity to a single processor in the current system
+            long lowestOneBit = Long.lowestOneBit(systemAffinity);
+            ULONG_PTR dwProcessAffinityMask = new ULONG_PTR(lowestOneBit);
+            assertTrue("Failed to set affinity",
+                    Kernel32.INSTANCE.SetProcessAffinityMask(pHandle, dwProcessAffinityMask));
+            assertTrue("Failed to get affinity masks.",
+                    Kernel32.INSTANCE.GetProcessAffinityMask(pHandle, pProcessAffinity, pSystemAffinity));
+            assertEquals("Process affinity doesn't match what was just set", lowestOneBit,
+                    pProcessAffinity.getValue().longValue());
+
+            // Now try to set affinity to an invalid processor
+            lowestOneBit = Long.lowestOneBit(~systemAffinity);
+            // In case we have exactly 64 processors we can't do this, otherwise...
+            if (lowestOneBit != 0) {
+                dwProcessAffinityMask = new ULONG_PTR(lowestOneBit);
+                assertFalse("Successfully set affinity when it should have failed",
+                        Kernel32.INSTANCE.SetProcessAffinityMask(pHandle, dwProcessAffinityMask));
+                assertEquals("Last error should be ERROR_INVALID_PARAMETER", WinError.ERROR_INVALID_PARAMETER,
+                        Kernel32.INSTANCE.GetLastError());
+            }
+
+            // Cleanup. Be nice and put affinity back where it started!
+            dwProcessAffinityMask = new ULONG_PTR(processAffinity);
+            assertTrue("Failed to restore affinity to original setting",
+                    Kernel32.INSTANCE.SetProcessAffinityMask(pHandle, dwProcessAffinityMask));
+        }
+    }
+
     public void testGetTempPath() {
         char[] buffer = new char[WinDef.MAX_PATH];
         assertTrue(Kernel32.INSTANCE.GetTempPath(new DWORD(WinDef.MAX_PATH), buffer).intValue() > 0);
@@ -993,7 +1049,7 @@ public class Kernel32Test extends TestCase {
             }
             FILE_ATTRIBUTE_TAG_INFO fati = new FILE_ATTRIBUTE_TAG_INFO(p);
             // New files have the archive bit
-            assertEquals(WinNT.FILE_ATTRIBUTE_ARCHIVE, fati.FileAttributes);
+            assertEquals(WinNT.FILE_ATTRIBUTE_ARCHIVE, fati.FileAttributes & WinNT.FILE_ATTRIBUTE_ARCHIVE);
 
             p = new Memory(FILE_ID_INFO.sizeOf());
             if (false == Kernel32.INSTANCE.GetFileInformationByHandleEx(hFile, WinBase.FileIdInfo, p, new DWORD(p.size()))) {
@@ -1124,6 +1180,28 @@ public class Kernel32Test extends TestCase {
         }
     }
 
+    public void testGetThreadList() throws IOException {
+        HANDLE threadEnumHandle = Kernel32.INSTANCE.CreateToolhelp32Snapshot(Tlhelp32.TH32CS_SNAPTHREAD,
+                new WinDef.DWORD(0));
+        assertFalse(WinBase.INVALID_HANDLE_VALUE.equals(threadEnumHandle));
+        try {
+            Tlhelp32.THREADENTRY32.ByReference threadEntry = new Tlhelp32.THREADENTRY32.ByReference();
+
+            assertTrue(Kernel32.INSTANCE.Thread32First(threadEnumHandle, threadEntry));
+
+            List<Integer> threadIdList = new ArrayList<Integer>();
+            threadIdList.add(threadEntry.th32ThreadID);
+
+            while (Kernel32.INSTANCE.Thread32Next(threadEnumHandle, threadEntry)) {
+                threadIdList.add(threadEntry.th32ThreadID);
+            }
+
+            assertTrue(threadIdList.size() > 4);
+        } finally {
+            Kernel32Util.closeHandle(threadEnumHandle);
+        }
+    }
+
     public final void testGetPrivateProfileInt() throws IOException {
         final File tmp = File.createTempFile("testGetPrivateProfileInt", "ini");
         tmp.deleteOnExit();
