@@ -1,28 +1,33 @@
 /* Copyright (c) 2007-2013 Timothy Wall, All Rights Reserved
  *
- * The contents of this file is dual-licensed under 2 
- * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+ * The contents of this file is dual-licensed under 2
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and
  * Apache License 2.0. (starting with JNA version 4.0.0).
- * 
- * You can freely decide which license you want to apply to 
+ *
+ * You can freely decide which license you want to apply to
  * the project.
- * 
+ *
  * You may obtain a copy of the LGPL License at:
- * 
+ *
  * http://www.gnu.org/licenses/licenses.html
- * 
+ *
  * A copy is also included in the downloadable source code package
  * containing JNA, in file "LGPL2.1".
- * 
+ *
  * You may obtain a copy of the Apache License at:
- * 
+ *
  * http://www.apache.org/licenses/
- * 
+ *
  * A copy is also included in the downloadable source code package
  * containing JNA, in file "AL2.0".
  */
 package com.sun.jna;
 
+import java.lang.annotation.Documented;
+import java.lang.annotation.ElementType;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
 import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
@@ -38,10 +43,13 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.WeakHashMap;
+import java.util.logging.Level;
+import java.util.logging.Logger;
 
 /**
  * Represents a native structure with a Java peer class.  When used as a
@@ -66,11 +74,12 @@ import java.util.WeakHashMap;
  * public.  If your structure is to have no fields of its own, it must be
  * declared abstract.
  * </p>
- * <p>You <em>must</em> define {@link #getFieldOrder} to return a List of
- * field names (Strings) indicating the proper order of the fields.  When
- * dealing with multiple levels of subclasses of Structure, you must add to
- * the list provided by the superclass {@link #getFieldOrder}
- * the fields defined in the current class.
+ * <p>You <em>must</em> annotate the class with {@link FieldOrder} or implement
+ * {@link #getFieldOrder}, whichever you choose it must contain the field names
+ * (Strings) indicating the proper order of the fields. If you chose to implement
+ * {@link #getFieldOrder} notice that when dealing with multiple levels of
+ * subclasses of Structure, you must add to the list provided by the superclass
+ * {@link #getFieldOrder} the fields defined in the current class.
  * </p>
  * <p>In the past, most VMs would return them in a predictable order, but the JVM
  * spec does not require it, so {@link #getFieldOrder} is now required to
@@ -102,6 +111,8 @@ import java.util.WeakHashMap;
  */
 public abstract class Structure {
 
+    private static final Logger LOG = Logger.getLogger(Structure.class.getName());
+
     /** Tagging interface to indicate the value of an instance of the
      * <code>Structure</code> type is to be used in function invocations rather
      * than its address.  The default behavior is to treat
@@ -484,8 +495,8 @@ public abstract class Structure {
             }
         }
         public Structure[] getElements() {
-			return elements;
-		}
+            return elements;
+        }
         @Override
         public int size() { return count; }
         @Override
@@ -593,11 +604,12 @@ public abstract class Structure {
      * @return return offset of the given field
      */
     protected int fieldOffset(String name) {
-	ensureAllocated();
-	StructField f = fields().get(name);
-        if (f == null)
+        ensureAllocated();
+        StructField f = fields().get(name);
+        if (f == null) {
             throw new IllegalArgumentException("No such field: " + name);
-	return f.offset;
+        }
+        return f.offset;
     }
 
     /** Force a read of the given field from native memory.  The Java field
@@ -662,7 +674,7 @@ public abstract class Structure {
      * @param address the native <code>struct *</code>
      * @return Updated <code>Structure.ByReference</code> object
      */
-    static Structure updateStructureByReference(Class<?> type, Structure s, Pointer address) {
+    static <T extends Structure> T updateStructureByReference(Class<T> type, T s, Pointer address) {
         if (address == null) {
             s = null;
         }
@@ -670,7 +682,7 @@ public abstract class Structure {
             if (s == null || !address.equals(s.getPointer())) {
                 Structure s1 = reading().get(address);
                 if (s1 != null && type.equals(s1.getClass())) {
-                    s = s1;
+                    s = (T) s1;
                     s.autoRead();
                 }
                 else {
@@ -865,20 +877,66 @@ public abstract class Structure {
         }
     }
 
-    /** Return this Structure's field names in their proper order.  For
-     * example,
+    /** Used to declare fields order as metadata instead of method.
+     * example:
      * <pre><code>
-     * protected List getFieldOrder() {
-     *     return Arrays.asList(new String[] { ... });
+     * // New
+     * {@literal @}FieldOrder({ "n", "s" })
+     * class Parent extends Structure {
+     *     public int n;
+     *     public String s;
+     * }
+     * {@literal @}FieldOrder({ "d", "c" })
+     * class Son extends Parent {
+     *     public double d;
+     *     public char c;
+     * }
+     * // Old
+     * class Parent extends Structure {
+     *     public int n;
+     *     public String s;
+     *     protected List<String> getFieldOrder() {
+     *         return Arrays.asList("n", "s");
+     *     }
+     * }
+     * class Son extends Parent {
+     *     public double d;
+     *     public char c;
+     *     protected List<String> getFieldOrder() {
+     *         List<String> fields = new LinkedList<String>(super.getFieldOrder());
+     *         fields.addAll(Arrays.asList("d", "c"));
+     *         return fields;
+     *     }
+     * }
+     * </code></pre>
+     */
+    @Documented
+    @Retention(RetentionPolicy.RUNTIME)
+    @Target(ElementType.TYPE)
+    public @interface FieldOrder {
+        String[] value();
+    }
+
+    /** Returns this Structure's field names in their proper order.<br>
+     *
+     * When defining a new {@link Structure} you shouldn't override this
+     * method, but use {@link FieldOrder} annotation to define your field
+     * order(this also works with inheritance)<br>
+     *
+     * If you want to do something non-standard you can override the method
+     * and define it as followed
+     * <pre><code>
+     * protected List<String> getFieldOrder() {
+     *     return Arrays.asList(...);
      * }
      * </code></pre>
      * <strong>IMPORTANT</strong>
      * When deriving from an existing Structure subclass, ensure that
      * you augment the list provided by the superclass, e.g.
      * <pre><code>
-     * protected List getFieldOrder() {
-     *     List fields = new ArrayList(super.getFieldOrder());
-     *     fields.addAll(Arrays.asList(new String[] { ... }));
+     * protected List<String> getFieldOrder() {
+     *     List<String> fields = new LinkedList<String>(super.getFieldOrder());
+     *     fields.addAll(Arrays.asList(...));
      *     return fields;
      * }
      * </code></pre>
@@ -888,17 +946,18 @@ public abstract class Structure {
      * guaranteed to be predictable.
      * @return ordered list of field names
      */
-    protected abstract List<String> getFieldOrder();
+    // TODO(idosu 28 Apr 2018): Maybe deprecate this method to let users know they should use @FieldOrder
+    protected List<String> getFieldOrder() {
+        List<String> fields = new LinkedList<String>();
+        for (Class<?> clazz = getClass(); clazz != Structure.class; clazz = clazz.getSuperclass()) {
+            FieldOrder order = clazz.getAnnotation(FieldOrder.class);
+            if (order != null) {
+                fields.addAll(0, Arrays.asList(order.value()));
+            }
+        }
 
-    /**
-     * Force a compile-time error on the old method of field definition
-     * @param fields ordered array of field names
-     * @deprecated Use the required method getFieldOrder() instead to
-     * indicate the order of fields in this structure.
-     */
-    @Deprecated
-    protected final void setFieldOrder(String[] fields) {
-        throw new Error("This method is obsolete, use getFieldOrder() instead");
+        // fields.isEmpty() can be true because it is check somewhere else
+        return Collections.unmodifiableList(fields);
     }
 
     /** Sort the structure fields according to the given array of names.
@@ -1007,7 +1066,10 @@ public abstract class Structure {
         if (fieldOrder.size() != flist.size() && flist.size() > 1) {
             if (force) {
                 throw new Error("Structure.getFieldOrder() on " + getClass()
-                                + " does not provide enough names [" + fieldOrder.size()
+                                + (fieldOrder.size() < flist.size()
+                                    ? " does not provide enough"
+                                    : " provides too many")
+                                + " names [" + fieldOrder.size()
                                 + "] ("
                                 + sort(fieldOrder)
                                 + ") to match declared fields [" + flist.size()
@@ -1053,7 +1115,7 @@ public abstract class Structure {
      * @param type Structure subclass to check
      * @return native size of the given Structure subclass
      */
-    static int size(Class<?> type) {
+    static int size(Class<? extends Structure> type) {
         return size(type, null);
     }
 
@@ -1062,7 +1124,7 @@ public abstract class Structure {
      * @param value optional instance of the given class
      * @return native size of the Structure subclass
      */
-    static int size(Class<?> type, Structure value) {
+    static <T extends Structure> int size(Class<T> type, T value) {
         LayoutInfo info;
         synchronized(layoutInfo) {
             info = layoutInfo.get(type);
@@ -1129,8 +1191,6 @@ public abstract class Structure {
         private int alignType = ALIGN_DEFAULT;
         private TypeMapper typeMapper;
         private boolean variable;
-        // For unions only, field on which the union FFI type info is based
-        private StructField typeInfoField;
     }
 
     private void validateField(String name, Class<?> type) {
@@ -1291,13 +1351,6 @@ public abstract class Structure {
 
             // Save the field in our list
             info.fields.put(structField.name, structField);
-
-            if (info.typeInfoField == null
-                || info.typeInfoField.size < structField.size
-                || (info.typeInfoField.size == structField.size
-                    && Structure.class.isAssignableFrom(structField.type))) {
-                info.typeInfoField = structField;
-            }
         }
 
         if (calculatedSize > 0) {
@@ -1319,6 +1372,7 @@ public abstract class Structure {
      * Initialize any null-valued fields that should have a non-null default
      * value.
      */
+    @SuppressWarnings("UseSpecificCatch")
     private void initializeFields() {
         // Get the full field list, don't care about sorting
         List<Field> flist = getFieldList();
@@ -1340,7 +1394,7 @@ public abstract class Structure {
         if (Structure.class.isAssignableFrom(type)
             && !(ByReference.class.isAssignableFrom(type))) {
             try {
-                value = newInstance(type, PLACEHOLDER_MEMORY);
+                value = newInstance((Class<? extends Structure>) type, PLACEHOLDER_MEMORY);
                 setFieldValue(field, value);
             }
             catch(IllegalArgumentException e) {
@@ -1411,15 +1465,15 @@ public abstract class Structure {
                  || Callback.class.isAssignableFrom(type)
                  || WString.class == type
                  || String.class == type) {
-            alignment = Pointer.SIZE;
+            alignment = Native.POINTER_SIZE;
         }
         else if (Structure.class.isAssignableFrom(type)) {
             if (ByReference.class.isAssignableFrom(type)) {
-                alignment = Pointer.SIZE;
+                alignment = Native.POINTER_SIZE;
             }
             else {
                 if (value == null)
-                    value = newInstance(type, PLACEHOLDER_MEMORY);
+                    value = newInstance((Class<? extends Structure>) type, PLACEHOLDER_MEMORY);
                 alignment = ((Structure)value).getStructAlignment();
             }
         }
@@ -1488,40 +1542,41 @@ public abstract class Structure {
         String contents = LS;
         if (!showContents) {
             contents = "...}";
-        }
-        else for (Iterator<StructField> i = fields().values().iterator(); i.hasNext();) {
-            StructField sf = i.next();
-            Object value = getFieldValue(sf.field);
-            String type = format(sf.type);
-            String index = "";
-            contents += prefix;
-            if (sf.type.isArray() && value != null) {
-                type = format(sf.type.getComponentType());
-                index = "[" + Array.getLength(value) + "]";
+        } else {
+            for (Iterator<StructField> i = fields().values().iterator(); i.hasNext();) {
+                StructField sf = i.next();
+                Object value = getFieldValue(sf.field);
+                String type = format(sf.type);
+                String index = "";
+                contents += prefix;
+                if (sf.type.isArray() && value != null) {
+                    type = format(sf.type.getComponentType());
+                    index = "[" + Array.getLength(value) + "]";
+                }
+                contents += String.format("  %s %s%s@0x%X", type, sf.name, index, sf.offset);
+                if (value instanceof Structure) {
+                    value = ((Structure)value).toString(indent + 1, !(value instanceof Structure.ByReference), dumpMemory);
+                }
+                contents += "=";
+                if (value instanceof Long) {
+                    contents += String.format("0x%08X", (Long) value);
+                }
+                else if (value instanceof Integer) {
+                    contents += String.format("0x%04X", (Integer) value);
+                }
+                else if (value instanceof Short) {
+                    contents += String.format("0x%02X", (Short) value);
+                }
+                else if (value instanceof Byte) {
+                    contents += String.format("0x%01X", (Byte) value);
+                }
+                else {
+                    contents += String.valueOf(value).trim();
+                }
+                contents += LS;
+                if (!i.hasNext())
+                    contents += prefix + "}";
             }
-            contents += String.format("  %s %s%s@0x%X", type, sf.name, index, sf.offset);
-            if (value instanceof Structure) {
-                value = ((Structure)value).toString(indent + 1, !(value instanceof Structure.ByReference), dumpMemory);
-            }
-            contents += "=";
-            if (value instanceof Long) {
-                contents += String.format("0x%08X", (Long) value);
-            }
-            else if (value instanceof Integer) {
-                contents += String.format("0x%04X", (Integer) value);
-            }
-            else if (value instanceof Short) {
-                contents += String.format("0x%02X", (Short) value);
-            }
-            else if (value instanceof Byte) {
-                contents += String.format("0x%01X", (Byte) value);
-            }
-            else {
-                contents += String.valueOf(value).trim();
-            }
-            contents += LS;
-            if (!i.hasNext())
-                contents += prefix + "}";
         }
         if (indent == 0 && dumpMemory) {
             final int BYTES_PER_ROW = 4;
@@ -1612,12 +1667,12 @@ public abstract class Structure {
      * @return equality result
      */
     public boolean dataEquals(Structure s, boolean clear) {
-	if (clear) {
-	    s.getPointer().clear(s.size());
-	    s.write();
-	    getPointer().clear(size());
-	    write();
-	}
+        if (clear) {
+            s.getPointer().clear(s.size());
+            s.write();
+            getPointer().clear(size());
+            write();
+        }
         byte[] data = s.getPointer().getByteArray(0, s.size());
         byte[] ref = getPointer().getByteArray(0, size());
         if (data.length == ref.length) {
@@ -1664,7 +1719,7 @@ public abstract class Structure {
      * @param f internal field representation
      * @return Native pointer to the corresponding type information
      */
-    Pointer getFieldTypeInfo(StructField f) {
+    FFIType getFieldTypeInfo(StructField f) {
         Class<?> type = f.type;
         Object value = getFieldValue(f.field);
         if (typeMapper != null) {
@@ -1681,7 +1736,7 @@ public abstract class Structure {
      * @return native type information for this structure.
      */
     Pointer getTypeInfo() {
-        Pointer p = getTypeInfo(this);
+        Pointer p = getTypeInfo(this).getPointer();
         cacheTypeInfo(p);
         return p;
     }
@@ -1747,7 +1802,7 @@ public abstract class Structure {
      * @param obj object to query
      * @return native pointer to type information
      */
-    static Pointer getTypeInfo(Object obj) {
+    static FFIType getTypeInfo(Object obj) {
         return FFIType.get(obj);
     }
 
@@ -1755,16 +1810,16 @@ public abstract class Structure {
      * #newInstance(Class,Pointer)}, except that it additionally calls
      * {@link #conditionalAutoRead()}.
      */
-    private static Structure newInstance(Class<?> type, long init) {
+    private static <T extends Structure> T newInstance(Class<T> type, long init) {
         try {
-            Structure s = newInstance(type, init == 0 ? PLACEHOLDER_MEMORY : new Pointer(init));
+            T s = newInstance(type, init == 0 ? PLACEHOLDER_MEMORY : new Pointer(init));
             if (init != 0) {
                 s.conditionalAutoRead();
             }
             return s;
         }
         catch(Throwable e) {
-            System.err.println("JNA: Error creating structure: " + e);
+            LOG.log(Level.WARNING, "JNA: Error creating structure", e);
             return null;
         }
     }
@@ -1776,12 +1831,12 @@ public abstract class Structure {
      * @return the new instance
      * @throws IllegalArgumentException if the instantiation fails
      */
-    public static Structure newInstance(Class<?> type, Pointer init) throws IllegalArgumentException {
+    public static <T extends Structure> T newInstance(Class<T> type, Pointer init) throws IllegalArgumentException {
         try {
-            Constructor<?> ctor = type.getConstructor(Pointer.class);
-            return (Structure)ctor.newInstance(init);
-        }
-        catch(NoSuchMethodException e) {
+            Constructor<T> ctor = getPointerConstructor(type);
+            if (ctor != null) {
+                return ctor.newInstance(init);
+            }
             // Not defined, fall back to the default
         }
         catch(SecurityException e) {
@@ -1797,52 +1852,43 @@ public abstract class Structure {
         }
         catch(InvocationTargetException e) {
             String msg = "Exception thrown while instantiating an instance of " + type;
-            e.printStackTrace();
             throw new IllegalArgumentException(msg, e);
         }
-        Structure s = newInstance(type);
+        T s = newInstance(type);
         if (init != PLACEHOLDER_MEMORY) {
             s.useMemory(init);
         }
         return s;
     }
 
-    /** Create a new Structure instance of the given type
+    /**
+     * Create a new Structure instance of the given type
      * @param type desired Structure type
      * @return the new instance
      * @throws IllegalArgumentException if the instantiation fails
      */
-    public static Structure newInstance(Class<?> type) throws IllegalArgumentException {
-        try {
-            Structure s = (Structure)type.newInstance();
-            if (s instanceof ByValue) {
-                s.allocateMemory();
-            }
-            return s;
-        }
-        catch(InstantiationException e) {
-            String msg = "Can't instantiate " + type;
-            throw new IllegalArgumentException(msg, e);
-        }
-        catch(IllegalAccessException e) {
-            String msg = "Instantiation of " + type
-                + " not allowed, is it public?";
-            throw new IllegalArgumentException(msg, e);
+    public static <T extends Structure> T newInstance(Class<T> type) throws IllegalArgumentException {
+        T s = Klass.newInstance(type);
+        if (s instanceof ByValue) {
+            s.allocateMemory();
         }
+        return s;
     }
 
-    /** Keep track of the largest aggregate field of the union to use for
-     * FFI type information.
-     * @return which field to use to obtain FFI type information
+    /**
+     * Returns a constructor for the given type with a single Pointer argument, null if no such constructor is found.
+     * @param type the class
+     * @param <T> the type
+     * @return a constructor with a single Pointer argument, null if none is found
      */
-    StructField typeInfoField() {
-        LayoutInfo info;
-        synchronized(layoutInfo) {
-            info = layoutInfo.get(getClass());
-        }
-        if (info != null) {
-            return info.typeInfoField;
+    private static <T> Constructor<T> getPointerConstructor(Class<T> type) {
+        for (Constructor constructor : type.getConstructors()) {
+            Class[] parameterTypes = constructor.getParameterTypes();
+            if (parameterTypes.length == 1 && parameterTypes[0].equals(Pointer.class)) {
+                return constructor;
+            }
         }
+
         return null;
     }
 
@@ -1862,10 +1908,12 @@ public abstract class Structure {
             return name + "@" + offset + "[" + size + "] (" + type + ")";
         }
     }
-    /** This class auto-generates an ffi_type structure appropriate for a given
+    /**
+     * This class auto-generates an ffi_type structure appropriate for a given
      * structure for use by libffi.  The lifecycle of this structure is easier
      * to manage on the Java side than in native code.
      */
+    @FieldOrder({ "size", "alignment", "type", "elements" })
     static class FFIType extends Structure {
         public static class size_t extends IntegerType {
             private static final long serialVersionUID = 1L;
@@ -1874,7 +1922,9 @@ public abstract class Structure {
             public size_t(long value) { super(Native.SIZE_T_SIZE, value); }
         }
 
-        private static final Map<Object, Object> typeInfoMap = new WeakHashMap<Object, Object>();
+        private static final Map<Class, FFIType> typeInfoMap = new WeakHashMap<Class, FFIType>();
+        private static final Map<Class, FFIType> unionHelper = new WeakHashMap<Class, FFIType>();
+        private static final Map<Pointer, FFIType> ffiTypeInfo = new HashMap<Pointer, FFIType>();
 
         // Native.initIDs initializes these fields to their appropriate
         // pointer values.  These are in a separate class from FFIType so that
@@ -1895,34 +1945,69 @@ public abstract class Structure {
             private static Pointer ffi_type_pointer;
         }
 
+        private static boolean isIntegerType(FFIType type) {
+            Pointer typePointer = type.getPointer();
+            return typePointer.equals(FFITypes.ffi_type_uint8)
+                || typePointer.equals(FFITypes.ffi_type_sint8)
+                || typePointer.equals(FFITypes.ffi_type_uint16)
+                || typePointer.equals(FFITypes.ffi_type_sint16)
+                || typePointer.equals(FFITypes.ffi_type_uint32)
+                || typePointer.equals(FFITypes.ffi_type_sint32)
+                || typePointer.equals(FFITypes.ffi_type_uint64)
+                || typePointer.equals(FFITypes.ffi_type_sint64)
+                || typePointer.equals(FFITypes.ffi_type_pointer);
+        }
+
+        private static boolean isFloatType(FFIType type) {
+            Pointer typePointer = type.getPointer();
+            return typePointer.equals(FFITypes.ffi_type_float)
+                || typePointer.equals(FFITypes.ffi_type_double);
+        }
+
         static {
             if (Native.POINTER_SIZE == 0)
                 throw new Error("Native library not initialized");
             if (FFITypes.ffi_type_void == null)
                 throw new Error("FFI types not initialized");
-            typeInfoMap.put(void.class, FFITypes.ffi_type_void);
-            typeInfoMap.put(Void.class, FFITypes.ffi_type_void);
-            typeInfoMap.put(float.class, FFITypes.ffi_type_float);
-            typeInfoMap.put(Float.class, FFITypes.ffi_type_float);
-            typeInfoMap.put(double.class, FFITypes.ffi_type_double);
-            typeInfoMap.put(Double.class, FFITypes.ffi_type_double);
-            typeInfoMap.put(long.class, FFITypes.ffi_type_sint64);
-            typeInfoMap.put(Long.class, FFITypes.ffi_type_sint64);
-            typeInfoMap.put(int.class, FFITypes.ffi_type_sint32);
-            typeInfoMap.put(Integer.class, FFITypes.ffi_type_sint32);
-            typeInfoMap.put(short.class, FFITypes.ffi_type_sint16);
-            typeInfoMap.put(Short.class, FFITypes.ffi_type_sint16);
-            Pointer ctype = Native.WCHAR_SIZE == 2
-                ? FFITypes.ffi_type_uint16 : FFITypes.ffi_type_uint32;
+            ffiTypeInfo.put(FFITypes.ffi_type_void, Structure.newInstance(FFIType.class, FFITypes.ffi_type_void));
+            ffiTypeInfo.put(FFITypes.ffi_type_float, Structure.newInstance(FFIType.class, FFITypes.ffi_type_float));
+            ffiTypeInfo.put(FFITypes.ffi_type_double, Structure.newInstance(FFIType.class, FFITypes.ffi_type_double));
+            ffiTypeInfo.put(FFITypes.ffi_type_longdouble, Structure.newInstance(FFIType.class, FFITypes.ffi_type_longdouble));
+            ffiTypeInfo.put(FFITypes.ffi_type_uint8, Structure.newInstance(FFIType.class, FFITypes.ffi_type_uint8));
+            ffiTypeInfo.put(FFITypes.ffi_type_sint8, Structure.newInstance(FFIType.class, FFITypes.ffi_type_sint8));
+            ffiTypeInfo.put(FFITypes.ffi_type_uint16, Structure.newInstance(FFIType.class, FFITypes.ffi_type_uint16));
+            ffiTypeInfo.put(FFITypes.ffi_type_sint16, Structure.newInstance(FFIType.class, FFITypes.ffi_type_sint16));
+            ffiTypeInfo.put(FFITypes.ffi_type_uint32, Structure.newInstance(FFIType.class, FFITypes.ffi_type_uint32));
+            ffiTypeInfo.put(FFITypes.ffi_type_sint32, Structure.newInstance(FFIType.class, FFITypes.ffi_type_sint32));
+            ffiTypeInfo.put(FFITypes.ffi_type_uint64, Structure.newInstance(FFIType.class, FFITypes.ffi_type_uint64));
+            ffiTypeInfo.put(FFITypes.ffi_type_sint64, Structure.newInstance(FFIType.class, FFITypes.ffi_type_sint64));
+            ffiTypeInfo.put(FFITypes.ffi_type_pointer, Structure.newInstance(FFIType.class, FFITypes.ffi_type_pointer));
+            for(FFIType f: ffiTypeInfo.values()) {
+                f.read();
+            }
+            typeInfoMap.put(void.class, ffiTypeInfo.get(FFITypes.ffi_type_void));
+            typeInfoMap.put(Void.class, ffiTypeInfo.get(FFITypes.ffi_type_void));
+            typeInfoMap.put(float.class, ffiTypeInfo.get(FFITypes.ffi_type_float));
+            typeInfoMap.put(Float.class, ffiTypeInfo.get(FFITypes.ffi_type_float));
+            typeInfoMap.put(double.class, ffiTypeInfo.get(FFITypes.ffi_type_double));
+            typeInfoMap.put(Double.class, ffiTypeInfo.get(FFITypes.ffi_type_double));
+            typeInfoMap.put(long.class, ffiTypeInfo.get(FFITypes.ffi_type_sint64));
+            typeInfoMap.put(Long.class, ffiTypeInfo.get(FFITypes.ffi_type_sint64));
+            typeInfoMap.put(int.class, ffiTypeInfo.get(FFITypes.ffi_type_sint32));
+            typeInfoMap.put(Integer.class, ffiTypeInfo.get(FFITypes.ffi_type_sint32));
+            typeInfoMap.put(short.class, ffiTypeInfo.get(FFITypes.ffi_type_sint16));
+            typeInfoMap.put(Short.class, ffiTypeInfo.get(FFITypes.ffi_type_sint16));
+            FFIType ctype = Native.WCHAR_SIZE == 2
+                ? ffiTypeInfo.get(FFITypes.ffi_type_uint16) : ffiTypeInfo.get(FFITypes.ffi_type_uint32);
             typeInfoMap.put(char.class, ctype);
             typeInfoMap.put(Character.class, ctype);
-            typeInfoMap.put(byte.class, FFITypes.ffi_type_sint8);
-            typeInfoMap.put(Byte.class, FFITypes.ffi_type_sint8);
-            typeInfoMap.put(Pointer.class, FFITypes.ffi_type_pointer);
-            typeInfoMap.put(String.class, FFITypes.ffi_type_pointer);
-            typeInfoMap.put(WString.class, FFITypes.ffi_type_pointer);
-            typeInfoMap.put(boolean.class, FFITypes.ffi_type_uint32);
-            typeInfoMap.put(Boolean.class, FFITypes.ffi_type_uint32);
+            typeInfoMap.put(byte.class, ffiTypeInfo.get(FFITypes.ffi_type_sint8));
+            typeInfoMap.put(Byte.class, ffiTypeInfo.get(FFITypes.ffi_type_sint8));
+            typeInfoMap.put(Pointer.class, ffiTypeInfo.get(FFITypes.ffi_type_pointer));
+            typeInfoMap.put(String.class, ffiTypeInfo.get(FFITypes.ffi_type_pointer));
+            typeInfoMap.put(WString.class, ffiTypeInfo.get(FFITypes.ffi_type_pointer));
+            typeInfoMap.put(boolean.class, ffiTypeInfo.get(FFITypes.ffi_type_uint32));
+            typeInfoMap.put(Boolean.class, ffiTypeInfo.get(FFITypes.ffi_type_uint32));
         }
         // From ffi.h
         private static final int FFI_TYPE_STRUCT = 13;
@@ -1932,57 +2017,104 @@ public abstract class Structure {
         public short type = FFI_TYPE_STRUCT;
         public Pointer elements;
 
-        private FFIType(Structure ref) {
+        public FFIType(FFIType reference) {
+            this.size = reference.size;
+            this.alignment = reference.alignment;
+            this.type = reference.type;
+            this.elements = reference.elements;
+        }
+
+        public FFIType() {}
+
+        public FFIType(Structure ref) {
             Pointer[] els;
             ref.ensureAllocated(true);
 
             if (ref instanceof Union) {
-                StructField sf = ((Union)ref).typeInfoField();
+                FFIType unionType = null;
+                int size = 0;
+                boolean hasInteger = false;
+                for (StructField sf : ref.fields().values()) {
+                    FFIType type = ref.getFieldTypeInfo(sf);
+                    if (isIntegerType(type)) {
+                        hasInteger = true;
+                    }
+                    if (unionType == null
+                        || size < sf.size
+                        || (size == sf.size
+                        && Structure.class.isAssignableFrom(sf.type))) {
+                        unionType = type;
+                        size = sf.size;
+                    }
+                }
+                if( (! Platform.isWindows()) && (
+                        (Platform.isIntel() && Platform.is64Bit())
+                        || (Platform.isARM())
+                    )) {
+                    // System V x86-64 ABI requires, that in a union aggregate,
+                    // that contains Integer and Double members, the parameters
+                    // must be passed in the integer registers. I.e. in the case
+                    // where the java side declares double and int members, the
+                    // wrong FFI Type would be found, because the doubles size
+                    // is larger than the int member, but the wrong parameter
+                    // passing method would be used.
+                    //
+                    // It was observed, that the same behaviour is visible on
+                    // arm/aarch64.
+                    if(hasInteger && isFloatType(unionType)) {
+                        unionType = new FFIType(unionType);
+                        if(unionType.size.intValue() == 4) {
+                            unionType.type = ffiTypeInfo.get(FFITypes.ffi_type_uint32).type;
+                        } else if (unionType.size.intValue() == 8) {
+                            unionType.type = ffiTypeInfo.get(FFITypes.ffi_type_uint64).type;
+                        }
+                        unionType.write();
+                    }
+                }
                 els = new Pointer[] {
-                    get(ref.getFieldValue(sf.field), sf.type),
+                    unionType.getPointer(),
                     null,
                 };
+                unionHelper.put(ref.getClass(), unionType);
             }
             else {
                 els = new Pointer[ref.fields().size() + 1];
                 int idx = 0;
                 for (StructField sf : ref.fields().values()) {
-                    els[idx++] = ref.getFieldTypeInfo(sf);
+                    els[idx++] = ref.getFieldTypeInfo(sf).getPointer();
                 }
             }
             init(els);
+            write();
         }
         // Represent fixed-size arrays as structures of N identical elements
-        private FFIType(Object array, Class<?> type) {
+        public FFIType(Object array, Class<?> type) {
             int length = Array.getLength(array);
             Pointer[] els = new Pointer[length+1];
-            Pointer p = get(null, type.getComponentType());
+            Pointer p = get(null, type.getComponentType()).getPointer();
             for (int i=0;i < length;i++) {
                 els[i] = p;
             }
             init(els);
+            write();
         }
 
-        @Override
-        protected List<String> getFieldOrder() {
-            return Arrays.asList(new String[] { "size", "alignment", "type", "elements" });
-        }
         private void init(Pointer[] els) {
-            elements = new Memory(Pointer.SIZE * els.length);
+            elements = new Memory(Native.POINTER_SIZE * els.length);
             elements.write(0, els, 0, els.length);
             write();
         }
 
         /** Obtain a pointer to the native FFI type descriptor for the given object. */
-        static Pointer get(Object obj) {
+        static FFIType get(Object obj) {
             if (obj == null)
-                return FFITypes.ffi_type_pointer;
+                return typeInfoMap.get(Pointer.class);
             if (obj instanceof Class)
                 return get(null, (Class<?>)obj);
             return get(obj, obj.getClass());
         }
 
-        private static Pointer get(Object obj, Class<?> cls) {
+        private static FFIType get(Object obj, Class<?> cls) {
             TypeMapper mapper = Native.getTypeMapper(cls);
             if (mapper != null) {
                 ToNativeConverter nc = mapper.getToNativeConverter(cls);
@@ -1991,27 +2123,24 @@ public abstract class Structure {
                 }
             }
             synchronized(typeInfoMap) {
-                Object o = typeInfoMap.get(cls);
-                if (o instanceof Pointer) {
-                    return (Pointer)o;
-                }
-                if (o instanceof FFIType) {
-                    return ((FFIType)o).getPointer();
+                FFIType o = typeInfoMap.get(cls);
+                if (o != null) {
+                    return o;
                 }
                 if ((Platform.HAS_BUFFERS && Buffer.class.isAssignableFrom(cls))
                     || Callback.class.isAssignableFrom(cls)) {
-                    typeInfoMap.put(cls, FFITypes.ffi_type_pointer);
-                    return FFITypes.ffi_type_pointer;
+                    typeInfoMap.put(cls, typeInfoMap.get(Pointer.class));
+                    return typeInfoMap.get(Pointer.class);
                 }
                 if (Structure.class.isAssignableFrom(cls)) {
-                    if (obj == null) obj = newInstance(cls, PLACEHOLDER_MEMORY);
+                    if (obj == null) obj = newInstance((Class<? extends Structure>) cls, PLACEHOLDER_MEMORY);
                     if (ByReference.class.isAssignableFrom(cls)) {
-                        typeInfoMap.put(cls, FFITypes.ffi_type_pointer);
-                        return FFITypes.ffi_type_pointer;
+                        typeInfoMap.put(cls, typeInfoMap.get(Pointer.class));
+                        return typeInfoMap.get(Pointer.class);
                     }
                     FFIType type = new FFIType((Structure)obj);
                     typeInfoMap.put(cls, type);
-                    return type.getPointer();
+                    return type;
                 }
                 if (NativeMapped.class.isAssignableFrom(cls)) {
                     NativeMappedConverter c = NativeMappedConverter.getInstance(cls);
@@ -2020,8 +2149,8 @@ public abstract class Structure {
                 if (cls.isArray()) {
                     FFIType type = new FFIType(obj, cls);
                     // Store it in the map to prevent premature GC of type info
-                    typeInfoMap.put(obj, type);
-                    return type.getPointer();
+                    typeInfoMap.put(cls, type);
+                    return type;
                 }
                 throw new IllegalArgumentException("Unsupported type " + cls);
             }
@@ -2135,7 +2264,7 @@ public abstract class Structure {
     /** Indicate whether the given Structure class can be created by JNA.
      * @param cls Structure subclass to check
      */
-    static void validate(Class<?> cls) {
+    static void validate(Class<? extends Structure> cls) {
         try {
             cls.getConstructor();
             return;
