@@ -1,22 +1,22 @@
-/**
- * The contents of this file is dual-licensed under 2 
- * alternative Open Source/Free licenses: LGPL 2.1 or later and 
+/*
+ * The contents of this file is dual-licensed under 2
+ * alternative Open Source/Free licenses: LGPL 2.1 or later and
  * Apache License 2.0. (starting with JNA version 4.0.0).
- * 
- * You can freely decide which license you want to apply to 
+ *
+ * You can freely decide which license you want to apply to
  * the project.
- * 
+ *
  * You may obtain a copy of the LGPL License at:
- * 
+ *
  * http://www.gnu.org/licenses/licenses.html
- * 
+ *
  * A copy is also included in the downloadable source code package
  * containing JNA, in file "LGPL2.1".
- * 
+ *
  * You may obtain a copy of the Apache License at:
- * 
+ *
  * http://www.apache.org/licenses/
- * 
+ *
  * A copy is also included in the downloadable source code package
  * containing JNA, in file "AL2.0".
  */
@@ -30,6 +30,8 @@ import org.reflections.util.ConfigurationBuilder;
 
 import java.lang.reflect.*;
 import java.util.ArrayList;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
@@ -56,7 +58,21 @@ public final class StructureFieldOrderInspector {
      */
     public static void batchCheckStructureGetFieldOrder(final Class<?> classDeclaredInSourceTreeToSearch,
                                                    final List<String> ignoreConstructorError) {
-        final Set<Class<? extends Structure>> classes = StructureFieldOrderInspector.findSubTypesOfStructure(classDeclaredInSourceTreeToSearch);
+        batchCheckStructureGetFieldOrder(classDeclaredInSourceTreeToSearch, ignoreConstructorError, false);
+    }
+
+    /**
+     * Search for Structure sub types in the source tree of the given class, and validate the getFieldOrder() method,
+     * and collects all errors into one exception.
+     *
+     * @param classDeclaredInSourceTreeToSearch a class who's source tree will be searched for Structure sub types.
+     * @param ignoreConstructorError list of classname prefixes for which to ignore construction errors.
+     * @param onlyInnerClasses limit scan to inner classes of the supplied class
+     */
+    public static void batchCheckStructureGetFieldOrder(final Class<?> classDeclaredInSourceTreeToSearch,
+                                                   final List<String> ignoreConstructorError,
+                                                   final boolean onlyInnerClasses) {
+        final Set<Class<? extends Structure>> classes = StructureFieldOrderInspector.findSubTypesOfStructure(classDeclaredInSourceTreeToSearch, onlyInnerClasses);
 
         final List<Throwable> problems = new ArrayList<Throwable>();
 
@@ -96,7 +112,7 @@ public final class StructureFieldOrderInspector {
     /**
      * Find all classes that extend {@link Structure}.
      */
-    public static Set<Class<? extends Structure>> findSubTypesOfStructure(final Class<?> classDeclaredInSourceTreeToSearch) {
+    public static Set<Class<? extends Structure>> findSubTypesOfStructure(final Class<?> classDeclaredInSourceTreeToSearch, boolean onlyInnerClasses) {
 
         // use: http://code.google.com/p/reflections/
 
@@ -105,9 +121,24 @@ public final class StructureFieldOrderInspector {
                 .setUrls(ClasspathHelper.forClass(classDeclaredInSourceTreeToSearch))
         );
 
-        return reflections.getSubTypesOf(Structure.class);
+        Set<Class<? extends Structure>> types = new HashSet<Class<? extends Structure>>(reflections.getSubTypesOf(Structure.class));
+        if(onlyInnerClasses) {
+            Iterator<Class<? extends Structure>> it = types.iterator();
+            while(it.hasNext()) {
+                if(! (it.next().getEnclosingClass() == classDeclaredInSourceTreeToSearch)) {
+                    it.remove();
+                }
+            }
+        }
+        return types;
     }
 
+    /**
+     * Find all classes that extend {@link Structure}.
+     */
+    public static Set<Class<? extends Structure>> findSubTypesOfStructure(final Class<?> classDeclaredInSourceTreeToSearch) {
+        return findSubTypesOfStructure(classDeclaredInSourceTreeToSearch, false);
+    }
 
     public static void checkMethodGetFieldOrder(final Class<? extends Structure> structureSubType,
                                                 final List<String> ignoreConstructorError) {
@@ -119,9 +150,6 @@ public final class StructureFieldOrderInspector {
             return;
         }
 
-        final Method methodGetFieldOrder = getMethodGetFieldOrder(structureSubType);
-
-
         if (Modifier.isAbstract(structureSubType.getModifiers())) {
             // do not try to construct abstract Structure sub types
             return;
@@ -161,58 +189,27 @@ public final class StructureFieldOrderInspector {
             throw new RuntimeException("Could not instantiate Structure sub type: " + structureSubType.getName(), e);
         }
 
-        if (!methodGetFieldOrder.isAccessible()) {
-            methodGetFieldOrder.setAccessible(true);
-        }
-        final List<?> methodCallFieldList;
-        try {
-            methodCallFieldList = (List<?>) methodGetFieldOrder.invoke(structure);
-        } catch (IllegalAccessException e) {
-            throw new RuntimeException("Could not invoke getFieldOrder() on Structure sub type: " + structureSubType.getName(), e);
-        } catch (InvocationTargetException e) {
-            throw new RuntimeException("Could not invoke getFieldOrder() on Structure sub type: " + structureSubType.getName(), e);
-        }
+        final List<String> methodCallFieldOrder = structure.getFieldOrder();
 
-        final Field[] actualFields = structureSubType.getFields(); // include fields from super classes
-        final List<String> actualFieldNames = new ArrayList<String>(actualFields.length);
+        final List<Field> actualFields = structure.getFieldList();
+        final List<String> actualFieldNames = new ArrayList<String>(actualFields.size());
         for (final Field field : actualFields) {
             // ignore static fields
             if (!Modifier.isStatic(field.getModifiers())) {
                 final String actualFieldName = field.getName();
-                if (!methodCallFieldList.contains(actualFieldName)) {
-                    throw new IllegalArgumentException(structureSubType.getName() + ".getFieldOrder() [" + methodCallFieldList
+                if (!methodCallFieldOrder.contains(actualFieldName)) {
+                    throw new IllegalArgumentException(structureSubType.getName() + ".getFieldOrder() [" + methodCallFieldOrder
                             + "] does not include declared field: " + actualFieldName);
                 }
                 actualFieldNames.add(actualFieldName);
             }
         }
 
-        for (final Object methodCallField : methodCallFieldList) {
+        for (final String methodCallField : methodCallFieldOrder) {
             if (!actualFieldNames.contains(methodCallField)) {
-                throw new IllegalArgumentException(structureSubType.getName() + ".getFieldOrder() [" + methodCallFieldList
+                throw new IllegalArgumentException(structureSubType.getName() + ".getFieldOrder() [" + methodCallFieldOrder
                         + "] includes undeclared field: " + methodCallField);
             }
         }
     }
-
-    /**
-     * Find the getFieldOrder() method in the given class, or any of it's parents.
-     * @param structureSubType a structure sub type
-     * @return the getFieldOrder() method found in the given class, or any of it's parents.
-     */
-    private static Method getMethodGetFieldOrder(Class<? extends Structure> structureSubType) {
-        final Method methodGetFieldOrder;
-        try {
-            methodGetFieldOrder = structureSubType.getDeclaredMethod("getFieldOrder", new Class[]{});
-        } catch (NoSuchMethodException e) {
-            if (structureSubType.getSuperclass() != null) {
-                // look for method in parent
-                return getMethodGetFieldOrder((Class<? extends Structure>) structureSubType.getSuperclass());
-            }
-            throw new IllegalArgumentException("The Structure sub type: " + structureSubType.getName()
-                    + " must define the method: getFieldOrder()."
-                    + " See the javadoc for Structure.getFieldOrder() for details.", e);
-        }
-        return methodGetFieldOrder;
-    }
 }
